@@ -1,17 +1,16 @@
 #!/bin/sh
 # the next line restarts using wish \
-exec wish8.2 "$0" "$@"
+exec wish8.4 "$0" "$@"
 
 # A cross-platform mixer application that adapts to the capabilities
 # of Snack on the machine it is run on.
 # Lots of functionality on Linux - play volume only on Windows, currently.
 
-package require -exact snack 2.0
+package require -exact snack 2.2
 
 proc Update {} {
-  global doMonitor
   snack::mixer update
-  if $doMonitor { after 100 Update }
+  if $::doMonitor { after 100 Update }
 }
 bind . <Configure> Update
 wm protocol . WM_DELETE_WINDOW exit
