@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1997-2003 Kare Sjolander <kare@speech.kth.se>
+ * Copyright (C) 1997-2004 Kare Sjolander <kare@speech.kth.se>
  *
  * This file is part of the Snack Sound Toolkit.
  * The latest version can be found at http://www.speech.kth.se/snack/
@@ -26,6 +26,17 @@
 extern "C" {
 #endif
 
+#ifdef __WIN32__
+  typedef __int32 int32_t;
+  typedef unsigned __int32 uint32_t;
+#else
+#  ifdef HAVE_STDINT_H
+#    include <stdint.h>
+#  else
+#    include <sys/types.h>
+#  endif
+#endif
+
 #if defined(MAC) || defined(MAC_OSX_TCL)
 #  include <string.h>
 #endif
@@ -242,9 +253,9 @@ extern int GetHeader(Sound *s, Tcl_Interp *interp, Tcl_Obj *obj);
 extern int PutHeader(Sound *s, Tcl_Interp *interp, int objc,
 		     Tcl_Obj *CONST objv[], int length);
 
-extern int WriteLELong(Tcl_Channel ch, long l);
+extern int WriteLELong(Tcl_Channel ch, int32_t l);
 
-extern int WriteBELong(Tcl_Channel ch, long l);
+extern int WriteBELong(Tcl_Channel ch, int32_t l);
 
 extern int SetFcname(Sound *s, Tcl_Interp *interp, Tcl_Obj *obj);
 
@@ -342,12 +353,24 @@ extern int convertCmd(Sound *s, Tcl_Interp *interp, int objc,
 		      Tcl_Obj *CONST objv[]);
 extern int dBPowerSpectrumCmd(Sound *s, Tcl_Interp *interp, int objc,
 			      Tcl_Obj *CONST objv[]);
+extern int powerSpectrumCmd(Sound *s, Tcl_Interp *interp, int objc,
+			    Tcl_Obj *CONST objv[]);
 extern int speaturesCmd(Sound *s, Tcl_Interp *interp, int objc,
 			Tcl_Obj *CONST objv[]);
 extern int alCmd(Sound *s, Tcl_Interp *interp, int objc,
 		 Tcl_Obj *CONST objv[]);
-extern int xoCmd(Sound *s, Tcl_Interp *interp, int objc,
+extern int vpCmd(Sound *s, Tcl_Interp *interp, int objc,
 		 Tcl_Obj *CONST objv[]);
+extern int joinCmd(Sound *s, Tcl_Interp *interp, int objc,
+		   Tcl_Obj *CONST objv[]);
+extern int fitCmd(Sound *s, Tcl_Interp *interp, int objc,
+		  Tcl_Obj *CONST objv[]);
+extern int inaCmd(Sound *s, Tcl_Interp *interp, int objc,
+		  Tcl_Obj *CONST objv[]);
+extern int lastIndexCmd(Sound *s, Tcl_Interp *interp, int objc,
+			Tcl_Obj *CONST objv[]);
+extern int stretchCmd(Sound *s, Tcl_Interp *interp, int objc,
+		      Tcl_Obj *CONST objv[]);
 extern int ocCmd(Sound *s, Tcl_Interp *interp, int objc,
 		 Tcl_Obj *CONST objv[]);
 extern int arCmd(Sound *s, Tcl_Interp *interp, int objc,
@@ -554,6 +577,12 @@ extern int Snack_arCmd(ClientData cdata, Tcl_Interp *interp, int objc,
 
 extern void Snack_arDeleteCmd(ClientData clientData);
 
+extern int isynCmd(ClientData cdata, Tcl_Interp *interp, int objc,
+		   Tcl_Obj *CONST objv[]);
+
+extern int osynCmd(ClientData cdata, Tcl_Interp *interp, int objc,
+		   Tcl_Obj *CONST objv[]);
+
 extern int WriteSound(writeSamplesProc *writeProc, Sound *s,
 		      Tcl_Interp *interp, Tcl_Channel ch, Tcl_Obj *obj,
 		      int startpos, int len);
@@ -564,7 +593,7 @@ extern void Snack_RemoveOptions(int objc, Tcl_Obj *CONST objv[],
 
 #ifndef MAC
 #  define PBSIZE 100000
-#  define NMAX 16384
+#  define NMAX 65536
 #else
 #  define PBSIZE 64000
 #  define NMAX 8192
@@ -581,6 +610,11 @@ extern void SnackPauseAudio();
 #define TCL_TELL Tcl_TellOld
 #endif
 
+#define SNACK_DB 4.34294481903251830000000 /*  = 10 / ln(10)  */
+#define SNACK_INTLOGARGMIN 1.0
+#define SNACK_CORRN (float) 138.308998699
+#define SNACK_CORR0 (float) 132.288396199
+
 /*
  * Include the public function declarations that are accessible via
  * the stubs table.
@@ -602,7 +636,7 @@ extern int SnackCloseFile(closeProc *closeProc, Sound *s, Tcl_Interp *interp,
 			  Tcl_Channel *ch);
 extern void Snack_ExecCallbacks(Sound *s, int flag);
 extern short Snack_SwapShort(short s);
-extern long Snack_SwapLong(long l);
+extern int32_t Snack_SwapLong(int32_t l);
 extern int Snack_ProgressCallback(Tcl_Obj *cmdPtr, Tcl_Interp *interp,
 				  char *type, double fraction);
 extern void Snack_DeleteSound(Sound *s);
@@ -616,6 +650,7 @@ extern void Snack_GetSoundData(Sound *s, int pos, void *buf, int nSamples);
 extern void Snack_InitWindow(float *win, int winlen, int fftlen, int type);
 extern int  Snack_InitFFT(int n);
 extern void Snack_DBPowerSpectrum(float *x);
+extern void Snack_PowerSpectrum(float *x);
 extern void Snack_CreateFilterType(Snack_FilterType *typePtr);
 extern int SaveSound(Sound *s, Tcl_Interp *interp, char *filename,
 		     Tcl_Obj *obj, int objc, Tcl_Obj *CONST objv[],
